@@ -6,6 +6,7 @@ import types
 
 from django.db.models import query, manager
 from django.db.models.fields import related
+from django.db.models.sql import where
 
 from django_group_access import registration
 from django_group_access.models import AccessManagerMixin, QuerySetMixin
@@ -43,18 +44,35 @@ def wrap_db_method(func, used_in_unique_check=False):
                 [t for t in traceback.extract_stack()
                 if t[2] == '_perform_unique_checks'])
 
+        # django 1.6 has a class that handles subqueries which we
+        # need to deal with slightly differently
+        is_subquery = hasattr(self, 'query_object')
+
         # if this queryset has already been filtered for access
         # control, let's not do it again.
         if apply_access_control:
-            if getattr(self, '_access_has_been_filtered', False):
+            if is_subquery:
+                if getattr(self.query_object, '_access_has_been_filtered', False):
+                    apply_access_control = False
+            elif getattr(self, '_access_has_been_filtered', False):
                 apply_access_control = False
 
         if apply_access_control:
-            queryset = self._filter_for_access_control()
+            if is_subquery:
+                queryset = self.query_object._filter_for_access_control()
+                self.query_object = queryset
+            else:
+                queryset = self._filter_for_access_control()
             queryset._access_has_been_filtered = True
         else:
-            queryset = self
-        return func(queryset, *args, **kwargs)
+            if is_subquery:
+                queryset = self.query_object
+            else:
+                queryset = self
+        if is_subquery:
+            return func(self, *args, **kwargs)
+        else:
+            return func(queryset, *args, **kwargs)
     return db_wrapper
 
 query.QuerySet.get = wrap_db_method(query.QuerySet.get)
@@ -65,6 +83,11 @@ query.QuerySet.count = wrap_db_method(query.QuerySet.count)
 query.QuerySet.in_bulk = wrap_db_method(query.QuerySet.in_bulk)
 query.QuerySet.__getitem__ = wrap_db_method(query.QuerySet.__getitem__)
 query.QuerySet._as_sql = wrap_db_method(query.QuerySet._as_sql)
+
+# django 1.6 subquery class
+if hasattr(where, 'SubqueryConstraint'):
+    where.SubqueryConstraint.as_sql = wrap_db_method(where.SubqueryConstraint.as_sql)
+
 # exists is used to do uniqueness checks when validating models.
 query.QuerySet.exists = wrap_db_method(
     query.QuerySet.exists, used_in_unique_check=True)
@@ -94,8 +117,14 @@ def wrap_get_query_set(func):
     return get_query_set_wrapper
 
 
-manager.Manager.get_query_set = wrap_get_query_set(
-    manager.Manager.get_query_set)
+if hasattr(manager.Manager, 'get_queryset'):
+    # django 1.6
+    manager.Manager.get_queryset = wrap_get_query_set(
+        manager.Manager.get_queryset)
+else:
+    # previous versions
+    manager.Manager.get_query_set = wrap_get_query_set(
+        manager.Manager.get_query_set)
 
 
 def wrap_query_set_iterator(func):
@@ -178,12 +207,20 @@ def wrap_descriptor_get(func, do_wrap_get_query_set=False):
             return obj
         if hasattr(instance, '_access_control_meta'):
             obj._access_control_meta = instance._access_control_meta
+
         if do_wrap_get_query_set and hasattr(obj, 'get_query_set'):
             if obj.get_query_set.im_func.__name__ != 'get_query_set_wrapper':
-                # only wrap it once
+                # only wrap it once (previous to django 1.6)
                 obj.get_query_set = types.MethodType(
                     wrap_related_manager_get_query_set(
                         obj.get_query_set), obj)
+
+        if do_wrap_get_query_set and hasattr(obj, 'get_queryset'):
+            if obj.get_queryset.im_func.__name__ != 'get_query_set_wrapper':
+                # only wrap it once (django 1.6)
+                obj.get_queryset = types.MethodType(
+                    wrap_related_manager_get_query_set(
+                        obj.get_queryset), obj)
         return obj
     return get_wrapper
 
